# datasets were read in from DICOM data, but there were oddities
# the DICOM data gave the wrong orientation 
#   (as determined by to3d, RSA instead of RAI)
# affine alignment of individual dataset to D99 template
#
# usage:
#    macaque_align.csh dset template_dset segmentation_dset
#
# example:
#    tcsh -x macaque_align.csh Puffin_3mar11_MDEFT_500um+orig \
#      atlas_dir/D99s_template+tlrc atlas_dir/D99_atlas+tlrc
#
if ("$#" !=  "3") then
   echo "usage:"
   echo "   macaque_align.csh dset template_dset segmentation_dset"
   echo "example:"
   echo " tcsh -x macaque_align.csh Puffin_3mar11_MDEFT_500um+orig \"
   echo "   atlas_dir/D99s_template+tlrc atlas_dir/D99_atlas_1.1_final+tlrc"
endif

setenv AFNI_COMPRESSOR GZIP

set dset = $1
set base = $2
set segset = $3
set baseprefix = `@GetAfniPrefix $base`

# optional resample to D99_template resolution and use that
# set finalmaster = `@GetAfniPrefix $base`
set finalmaster = $dset

# set dset = Puffin_3mar11_MDEFT_500um+orig
# set base = atlasdir/D99s_template+tlrc.
# set segset = atlasdir/D99_atlas+tlrc

set dsetprefix = `@GetAfniPrefix $dset`
set origdsetprefix = $dsetprefix
set segsetprefix = `@GetAfniPrefix $segset`
set segsetdir = `dirname $segset`

@Align_Centers -base $base -dset $dset

set dset = ${dsetprefix}_shft+orig
set dsetprefix = `@GetAfniPrefix $dset`

# skip ahead line if you want to bypass anything
# goto segsurfaces
rm  ${dsetprefix}_al2std+tlrc.*

# do affine alignment with lpa cost
# using dset as dset2 input and the base as dset1 
# (the base and source are treated differently 
# by align_epi_anats resampling and by 3dAllineate)
#  ******* feature size 4 mm is main difference from NIMH macaques  ******************
align_epi_anat.py -dset1 $base -dset2 $dset -dset2to1 -feature_size 4 \
   -suffix _al2std -dset1_strip None -dset2_strip None -giant_move \
   -master_dset2 BASE -overwrite

## put affine aligned data on template grid
# giant_move does something like
# 3dAllineate -1Dmatrix_apply ${dsetprefix}_al2std_mat.aff12.1D \
#    -prefix ${dsetprefix}_aff -base $base -master BASE         \
#    -source $dset -overwrite

# using 3dQwarp here instead of auto_warp.py (better gzip control)
3dQwarp -base $base -source  ${dsetprefix}_al2std+tlrc  \
  -prefix ${dsetprefix}_warp2std -overwrite -resample

apply_warps:
# the awpy has the result dataset, copy the warped data and the warp
# cp awpy_${dsetprefix}/${dsetprefix}_aff.aw.nii ./${dsetprefix}_warp2std.nii
# cp awpy_${dsetprefix}/anat.un.qw_WARP.nii ${dsetprefix}_WARP.nii

# if the datasets are compressed, then copy those instead
# cp awpy_${dsetprefix}/${dsetprefix}_aff.aw.nii.gz ./${dsetprefix}_warp2std.nii.gz
# cp awpy_${dsetprefix}/anat.un.qw_WARP.nii.gz  ./${dsetprefix}_WARP.nii.gz

# compress these copies (if not already compressed)
# gzip -f ${dsetprefix}_warp2std.nii ${dsetprefix}_WARP.nii

cat_matvec -ONELINE ${dsetprefix}_al2std_mat.aff12.1D -I >! ${dsetprefix}_inv.aff12.1D
# warp segmentation from atlas back to the original macaque space 
#  of the input dataset (compose overall warp when applying)
#  note - if transforming other datasets like the template
#    back to the same native space, it will be faster to compose
#    the warp separately with 3dNwarpCat or 3dNwarpCalc rather
#    than composing it for each 3dNwarpApply
3dNwarpApply -ainterp NN -short -overwrite -nwarp \
   "${dsetprefix}_inv.aff12.1D INV(${dsetprefix}_warp2std_WARP+tlrc)"  -overwrite \
   -source $segset -master ${dsetprefix}+orig -prefix __tmp_${dsetprefix}_seg

# put back in non-shifted version (really native space) 
@Align_Centers -base ${finalmaster} -dset  __tmp_${dsetprefix}_seg+orig. -no_cp

# change the datum type to byte to save space
# this step also gets rid of the shift transform in the header
3dcalc -a __tmp_${dsetprefix}_seg+orig -expr a -datum byte -nscale \
   -overwrite -prefix ${origdsetprefix}_seg

# copy segmentation information from atlas to this native-space
#   segmentation dataset and mark to be shown with integer colormap
3drefit -cmap INT_CMAP ${origdsetprefix}_seg+orig
3drefit -copytables $segset ${origdsetprefix}_seg+orig

# create transformed D99 template in this macaque's native space
# this dataset is useful for visualization
3dNwarpApply -overwrite -short -nwarp \
   "${dsetprefix}_inv.aff12.1D INV(${dsetprefix}_warp2std_WARP+tlrc)" \
   -source $base -master ${dsetprefix}+orig -prefix __tmp_D99_in_${dsetprefix}_native -overwrite
@Align_Centers -base ${finalmaster} -dset __tmp_D99_in_${dsetprefix}_native+orig. -no_cp
# change the datum type to byte to save space
# this step also gets rid of the shift transform in the header
3dcalc -a __tmp_D99_in_${dsetprefix}_native+orig -expr a -datum byte -nscale \
   -prefix ${baseprefix}_in_${origdsetprefix}_native -overwrite

apply_warps2:
# Warp the right side of the atlas separately
# easier than figuring out right from left in native space directly
3dcalc -a $base -prefix __tmp_right_seg -RAI -expr 'a*step(-x)' -overwrite
3dNwarpApply -ainterp NN -short -overwrite -nwarp \
   "${dsetprefix}_inv.aff12.1D INV(${dsetprefix}_warp2std_WARP+tlrc)"  -overwrite \
   -source __tmp_right_seg+tlrc \
   -master ${dsetprefix}+orig -prefix __tmp_${dsetprefix}_right_seg

# put back in non-shifted version (really native space) 
@Align_Centers -base ${finalmaster} -dset  __tmp_${dsetprefix}_right_seg+orig. -no_cp

# change the datum type to byte to save space
# this step also gets rid of the shift transform in the header
3dcalc -a __tmp_${dsetprefix}_right_seg+orig -expr a -datum byte -nscale \
   -overwrite -prefix ${origdsetprefix}_right_seg  -overwrite

# copy segmentation information from atlas to this native-space
#   segmentation dataset and mark to be shown with integer colormap
3drefit -cmap INT_CMAP ${origdsetprefix}_right_seg+orig
3drefit -copytables $segset ${origdsetprefix}_right_seg+orig

segsurfaces:
# for  oblique data, suma, unlike afni, uses the real xyz coordinates
#  instead of the cardinal equivalents
# 3drefit -atrcopy $finalmaster IJK_TO_DICOM_REAL ${origdsetprefix}_seg+orig
# 3drefit -atrcopy $finalmaster IJK_TO_DICOM_REAL ${origdsetprefix}_right_seg+orig
# Instead of doing the copies, work around the problem by creating an "unoblique" copy
#   of original and remove obliquity from segmentation datasets
3dcopy $finalmaster ${origdsetprefix}_unob
3drefit -deoblique ${origdsetprefix}_unob+orig
3drefit -deoblique ${origdsetprefix}_seg+orig
3drefit -deoblique ${origdsetprefix}_right_seg+orig

# create surfaces for all regions in atlas, but now in native space
rm -rf surfaces
mkdir surfaces
cd surfaces
IsoSurface -isorois+dsets -o native.gii -input ../${origdsetprefix}_seg+orig -noxform \
   -Tsmooth 0.1 100 -remesh 0.1
cd ..

rm -rf right_surfaces
mkdir right_surfaces
cd right_surfaces
IsoSurface -isorois+dsets -o native.gii -input ../${origdsetprefix}_right_seg+orig -noxform \
   -Tsmooth 0.1 100 -remesh 0.1
cd ..

anatsurface:

# "carve" out D99 surface in native space to use as representative surface
#  using anisotropic smoothing
# could use skullstripped original instead
3danisosmooth -prefix ${baseprefix}_in_${origdsetprefix}_aniso -overwrite\
   -3D -iters 6 -matchorig ${baseprefix}_in_${origdsetprefix}_native+orig
# also remove any  small clusters for surface generation (threshold here is specific so may need tweaking)
3dclust -1Dformat -nosum -1dindex 0 -1tindex 0 -2thresh -57.2 57.2 \
   -savemask ${baseprefix}_in_${origdsetprefix}_aniso_clust -overwrite \
   -dxyz=1 1.01 20000 \
   ${baseprefix}_in_${origdsetprefix}_aniso+orig

# for  oblique data, suma, unlike afni, uses the real xyz coordinates
#  instead of the cardinal equivalents
# 3drefit -atrcopy $finalmaster IJK_TO_DICOM_REAL \
#    ${baseprefix}_in_${origdsetprefix}_aniso+orig

IsoSurface -isorange 1 255 -overwrite \
   -input ${baseprefix}_in_${origdsetprefix}_aniso_clust+orig. \
   -o  ${baseprefix}_in_${origdsetprefix}_aniso.gii -overwrite -noxform -Tsmooth 0.1 20

echo "Dataset output:"
echo "   segmentation in native space:  ${origdsetprefix}_seg+orig"
echo "   base template in native space: ${baseprefix}_in_${origdsetprefix}+orig"
echo "   surfaces in native space in surfaces directory"
echo "   right-side surfaces in native space in right_surfaces directory"
echo   
echo "to show surfaces in suma, use a command like this:"
echo "    suma -onestate -i surfaces/native*.gii -vol ${origdsetprefix}_unob+orig -sv  ${baseprefix}_in_${origdsetprefix}_native+orig"
echo "or to show the template transformed to native space, use this:"
echo "    suma -onestate -i surfaces/native*.gii -vol ${origdsetprefix}_unob+orig -sv  ${baseprefix}_in_${origdsetprefix}_native+orig"
echo
echo "or show ${baseprefix} template as a *surface* in native space with segmentation volume, use this command"
echo "    suma -onestate -anatomical -i ${baseprefix}_in_${origdsetprefix}_aniso.gii -vol ${origdsetprefix}_seg+orig  -sv ${origdsetprefix}_seg+orig"
echo

